@@ -11,7 +11,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized );
 
-our $VERSION = '1.699_006';
+our $VERSION = '1.699_007';
 
 ## no critic (Subroutines::ProhibitExplicitReturnUndef)
 
@@ -123,8 +123,9 @@ sub new {
    MCE::Util::_sock_pair($_Q, qw(_qr_sock _qw_sock));
    MCE::Util::_sock_pair($_Q, qw(_ar_sock _aw_sock)) if $_Q->{_await};
 
-   syswrite $_Q->{_qw_sock}, $LF
-      if (exists $_argv{queue} && scalar @{ $_argv{queue} });
+   if (exists $_argv{queue} && scalar @{ $_argv{queue} }) {
+      1 until syswrite $_Q->{_qw_sock}, $LF;
+   }
 
    return $_Q;
 }
@@ -151,8 +152,9 @@ sub clear {
       warn "Queue: (clear) is not allowed for fast => 1\n";
    }
    else {
-      sysread $_Q->{_qr_sock}, $_next, 1 if ($_Q->_has_data());
-
+      if ($_Q->_has_data()) {
+         1 until sysread $_Q->{_qr_sock}, $_next, 1;
+      }
       %{ $_Q->{_datp} } = ();
       @{ $_Q->{_datq} } = ();
       @{ $_Q->{_heap} } = ();
@@ -168,8 +170,9 @@ sub enqueue {
 
    return unless (scalar @_);
 
-   syswrite $_Q->{_qw_sock}, $LF
-      if (!$_Q->{_nb_flag} && !$_Q->_has_data());
+   if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
+      1 until syswrite $_Q->{_qw_sock}, $LF;
+   }
 
    push @{ $_Q->{_datq} }, @_;
 
@@ -186,8 +189,9 @@ sub enqueuep {
 
    return unless (scalar @_);
 
-   syswrite $_Q->{_qw_sock}, $LF
-      if (!$_Q->{_nb_flag} && !$_Q->_has_data());
+   if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
+      1 until syswrite $_Q->{_qw_sock}, $LF;
+   }
 
    $_Q->_enqueuep($_p, @_);
 
@@ -200,7 +204,7 @@ sub dequeue {
    my ($_Q, $_cnt) = @_;
    my (@_items, $_buf, $_next, $_pending);
 
-   sysread $_Q->{_qr_sock}, $_next, 1;  # block
+   1 until sysread $_Q->{_qr_sock}, $_next, 1;  # block
 
    if (defined $_cnt && $_cnt ne '1') {
       @_items = $_Q->_dequeue($_cnt);
@@ -215,9 +219,9 @@ sub dequeue {
          $_pending = int($_pending / $_cnt) if (defined $_cnt);
          if ($_pending) {
             $_pending = MAX_DQ_DEPTH if ($_pending > MAX_DQ_DEPTH);
-            syswrite $_Q->{_qw_sock}, $LF for (1 .. $_pending);
+            for (1 .. $_pending) { 1 until syswrite $_Q->{_qw_sock}, $LF }
          }
-         $_Q->{_dsem}  = $_pending;
+         $_Q->{_dsem} = $_pending;
       }
       else {
          $_Q->{_dsem} -= 1;
@@ -225,7 +229,7 @@ sub dequeue {
    }
    else {
       ## Otherwise, never to exceed one byte in the channel
-      syswrite $_Q->{_qw_sock}, $LF if ($_Q->_has_data());
+      if ($_Q->_has_data()) { 1 until syswrite $_Q->{_qw_sock}, $LF }
    }
 
    $_Q->{_nb_flag} = 0;
@@ -276,8 +280,9 @@ sub insert {
 
    return unless (scalar @_);
 
-   syswrite $_Q->{_qw_sock}, $LF
-      if (!$_Q->{_nb_flag} && !$_Q->_has_data());
+   if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
+      1 until syswrite $_Q->{_qw_sock}, $LF;
+   }
 
    if (abs($_i) > scalar @{ $_Q->{_datq} }) {
       if ($_i >= 0) {
@@ -319,8 +324,9 @@ sub insertp {
 
    return unless (scalar @_);
 
-   syswrite $_Q->{_qw_sock}, $LF
-      if (!$_Q->{_nb_flag} && !$_Q->_has_data());
+   if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
+      1 until syswrite $_Q->{_qw_sock}, $LF;
+   }
 
    if (exists $_Q->{_datp}->{$_p} && scalar @{ $_Q->{_datp}->{$_p} }) {
 
@@ -597,7 +603,7 @@ MCE::Shared::Queue - Hybrid-queue helper class
 
 =head1 VERSION
 
-This document describes MCE::Shared::Queue version 1.699_006
+This document describes MCE::Shared::Queue version 1.699_007
 
 =head1 SYNOPSIS
 
